use std::fs::{File, OpenOptions};
use std::io;
use std::os::fd::OwnedFd;
use std::os::unix::fs::OpenOptionsExt;
use std::path::Path;

use input::{Libinput, LibinputInterface};
use input::Event;
use input::Event::{Keyboard, Pointer};
use input::event::keyboard::{KeyboardEventTrait, KeyState};
use input::event::{KeyboardEvent, PointerEvent};
use input::event::PointerEvent::{Button, Motion};
use libc::{O_RDONLY, O_RDWR, O_WRONLY};
use crate::input::events::keyboard::KvmKeyboardEvent;
use crate::input::events::KvmEvent;
use crate::input::events::pointer::{KvmPointerButtonEvent, KvmPointerMotionEvent};
use crate::input::mappings::FromLibinputEvent;

mod events;
mod mappings;

const UDEV_SEAT: &str = "seat0";

struct Interface;

impl LibinputInterface for Interface {
    fn open_restricted(&mut self, path: &Path, flags: i32) -> Result<OwnedFd, i32> {
        OpenOptions::new()
            .custom_flags(flags)
            .read((flags & O_RDONLY == O_RDONLY) | (flags & O_RDWR == O_RDWR))
            .write((flags & O_WRONLY == O_WRONLY) | (flags & O_RDWR == O_RDWR))
            .open(path)
            .map(|file| file.into())
            .map_err(|err| err.raw_os_error().unwrap())
    }

    fn close_restricted(&mut self, fd: OwnedFd) {
        File::from(fd);
    }
}

pub fn listen_inputs() -> io::Result<()> {
    let input = prepare_libinput();
    read_inputs(input)
}

fn prepare_libinput() -> Libinput {
    let mut input = Libinput::new_with_udev(Interface);
    input.udev_assign_seat(UDEV_SEAT).unwrap();
    input
}

fn read_inputs(mut input: Libinput) -> io::Result<()> {
    loop {
        input.dispatch().unwrap();
        for event in &mut input {
            let mapping_result = KvmEvent::from_libinput(event);
            if mapping_result.is_err() {
                println!("Error: {:?}", &mapping_result.err());
            } else {
                let kvm_event = mapping_result.unwrap();
                match kvm_event {
                    KvmEvent::Keyboard(event) => println!("Key event: {} ({})", event.key_code, event.state),
                    _ => {}
                }
            }
        }
    }
}